@@ -124,6 +124,20 @@ Available representations: text/plain, application/json`
 	}
 }
 
+func TestUnsupportedMedia_Issue492(t *testing.T) {
+	tearDown()
+	Add(newPostTestService())
+	for _, method := range []string{"POST", "PUT", "PATCH"} {
+		httpRequest, _ := http.NewRequest(method, "http://here.com/test", nil)
+		httpRequest.Header.Set("Accept", "application/json")
+		httpWriter := httptest.NewRecorder()
+		DefaultContainer.dispatch(httpWriter, httpRequest)
+		if 415 != httpWriter.Code {
+			t.Errorf("[%s] 415 expected got %d", method, httpWriter.Code)
+		}
+	}
+}
+
 func TestSelectedRoutePath_Issue100(t *testing.T) {
 	tearDown()
 	Add(newSelectedRouteTestingService())
@@ -313,6 +327,69 @@ func TestOptionsShortcut(t *testing.T) {
 	}
 }
 
+func TestClientWithAndWithoutTrailingSlashOld(t *testing.T) {
+	tearDown()
+	MergePathStrategy = TrimSlashStrategy
+	ws := new(WebService).Path("/test")
+	ws.Route(ws.PUT("/").To(return200))
+	Add(ws)
+
+	for _, tt := range []struct {
+		url      string
+		wantCode int
+	}{
+		// TrimSlashStrategy
+		{url: "http://here.com/test", wantCode: 404},
+		{url: "http://here.com/test/", wantCode: 200},
+		// PathJoinStrategy
+		//{url: "http://here.com/test", wantCode: 200},
+		//{url: "http://here.com/test/", wantCode: 404},
+	} {
+		t.Run(tt.url, func(t *testing.T) {
+			httpRequest, _ := http.NewRequest("PUT", tt.url, nil)
+			httpRequest.Header.Set("Accept", "*/*")
+			httpWriter := httptest.NewRecorder()
+			// override the default here
+			DefaultContainer.DoNotRecover(false)
+			DefaultContainer.dispatch(httpWriter, httpRequest)
+			if tt.wantCode != httpWriter.Code {
+				t.Errorf("Expected %d, got %d", tt.wantCode, httpWriter.Code)
+			}
+		})
+	}
+}
+
+func TestClientWithAndWithoutTrailingSlash(t *testing.T) {
+	tearDown()
+	ws := new(WebService).Path("/test")
+	ws.Route(ws.PUT("/").To(return200))
+	Add(ws)
+
+	for _, tt := range []struct {
+		url      string
+		wantCode int
+	}{
+		// TrimSlashStrategy
+		//{url: "http://here.com/test", wantCode: 404},
+		//{url: "http://here.com/test/", wantCode: 200},
+		// PathJoinStrategy
+		{url: "http://here.com/test", wantCode: 200},
+		{url: "http://here.com/test/", wantCode: 404},
+	} {
+		t.Run(tt.url, func(t *testing.T) {
+			httpRequest, _ := http.NewRequest("PUT", tt.url, nil)
+			httpRequest.Header.Set("Accept", "*/*")
+			httpWriter := httptest.NewRecorder()
+			// override the default here
+			DefaultContainer.DoNotRecover(false)
+			DefaultContainer.dispatch(httpWriter, httpRequest)
+			if tt.wantCode != httpWriter.Code {
+				t.Errorf("Expected %d, got %d", tt.wantCode, httpWriter.Code)
+			}
+		})
+	}
+}
+
 func newPanicingService() *WebService {
 	ws := new(WebService).Path("")
 	ws.Route(ws.GET("/fire").To(doPanic))
@@ -376,6 +453,16 @@ func newPostNoConsumesService() *WebService {
 	return ws
 }
 
+func newPostTestService() *WebService {
+	ws := new(WebService).Path("")
+	ws.Consumes("application/json")
+	ws.Produces("application/json")
+	ws.Route(ws.POST("/test").To(doNothing))
+	ws.Route(ws.PUT("/test").To(doNothing))
+	ws.Route(ws.PATCH("/test").To(doNothing))
+	return ws
+}
+
 func newSelectedRouteTestingService() *WebService {
 	ws := new(WebService).Path("")
 	ws.Route(ws.GET(pathGetFriends).To(selectedRouteChecker))
